pub struct ArcRef<'a, T: 'static> { /* private fields */ }Expand description
A type that represents &'a Arc<T>.
Note there is no verification-only permission field, because ArcRef uses Arc instead of a raw pointer internally.
Implementations
Sourceimpl<'a, T> ArcRef<'a, T>
impl<'a, T> ArcRef<'a, T>
Sourcepub exec fn deref_target(&self) -> ret : &'a T
pub exec fn deref_target(&self) -> ret : &'a T
ensures
*ret == self@,Dereferences self to get a reference to T with the lifetime 'a.
VERUS LIMITATION: The code includes a cast from &T to *const T, which is not specified yet in Verus.
This is also a nontrivial use case that extends the lifetime of the reference.